#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
import nardascripting.base.toolbox as tb
import time
import sys, struct, socket
import errno


class SharkWOL(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings:
        # Please adapt the following lines of code according to your script.
        # -------------------------------------------------------------------
        self._tab_name = "nsltools"  # Tab text
        self._scr_title = "SignalShark WOL V1.0"
        self._scr_description = "Starts a SignalShark via WOL"
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")  # Optional Script Icon path
        self._list_prio = 9  # Defines the position in the script list (0 = first item)
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL  # Sets the behavior after script execution

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # Get UDP broadcast address:
        # -------------------------------------------------------------------------------
        udp_addr = self.config.get("udp_addr", "192.168.178.255")
        dlg = wx.TextEntryDialog(self.frm_dlghelper, "Enter UDP broadcast address", "UDP broadcast address")
        dlg.SetValue(udp_addr)
        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            udp_addr = dlg.GetValue()
            self.config["udp_addr"] = udp_addr
        else:
            dlg.Destroy()
            return
        dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore

        # Get MAC address of SignalShark device:
        # -------------------------------------------------------------------------------
        mac = self.config.get("mac", "00:D0:93:2E:0E:CF")
        dlg = wx.TextEntryDialog(self.frm_dlghelper, "Enter the MAC address of the SignalShark device to be woken up:",
                                 "MAC address")
        dlg.SetValue(mac)
        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            mac = dlg.GetValue()
            self.config["mac"] = mac
        else:
            dlg.Destroy()
            return
        dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore

        # Wake up SignalShark:
        # -------------------------------------------------------------------------------
        wol_port = 40000
        self.wake_on_lan(mac, udp_addr, wol_port)


    def wake_on_lan(self, mac: str, udp_addr: str, port: int):
        # Construct 6 byte hardware address
        mac_parts = mac.split(':')
        if len(mac_parts) != 6:
            self.MessageBoxModal("MAC address should be written as 00:D0:93:2E:0E:CF",
                                 "Wrong MAC address", wx.OK | wx.ICON_INFORMATION)
            return

        mac_hex = struct.pack('BBBBBB',
                          int(mac_parts[0],16),
                          int(mac_parts[1],16),
                          int(mac_parts[2],16),
                          int(mac_parts[3],16),
                          int(mac_parts[4],16),
                          int(mac_parts[5],16))

        # Build magic packet
        mpackage = b'\xFF' * 6 + mac_hex * 16

        # Send packet to broadcast address using UDP
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        s.sendto(mpackage,(udp_addr, port))
        s.close()

